@@ -1,4 +1,7 @@
-from .aggregations import Aggregate, GroupBy, Reduce, ReduceFuncs
+"""
+The main module exposing public API (via conversion object)
+"""
+from .aggregations import Aggregate, BaseReducer, GroupBy, Reduce, ReduceFuncs
 from .base import (
     And,
     BaseConversion,
@@ -36,11 +39,35 @@ __all__ = ["conversion"]
 
 
 class _Conversion:
-    ConversionException = ConversionException
-    BaseConversion = BaseConversion
-    OptionsCtx = ConverterOptionsCtx
+    """The object, which exposes public API
+
+    .. code-block:: python
+
+      from convtools import conversion as c
+
+      convert = c.aggregate(
+          c.ReduceFuncs.DictSum(
+              c.item("name"),
+              c.item("value")
+          )
+      ).gen_converter(debug=True)
+
+      assert convert([
+          {"name": "Bob", "value": 10},
+          {"name": "Bob", "value": 7},
+          {"name": "Ron", "value": 3},
+      ]) == {'Bob': 17, 'Ron': 3}
+
+    """
+
+    ConversionException = ConversionException  # pylint: disable=invalid-name
+    BaseConversion = BaseConversion  # pylint: disable=invalid-name
+    OptionsCtx = ConverterOptionsCtx  # pylint: disable=invalid-name
+
+    ReduceFuncs = ReduceFuncs  # pylint: disable=invalid-name
+    #: Shortcut to `Mutations`
+    Mut = Mutations  # pylint: disable=invalid-name
 
-    ReduceFuncs = ReduceFuncs
     and_ = And
     or_ = Or
     not_ = Not
@@ -72,14 +99,18 @@ class _Conversion:
     dict_comp = DictComp
 
     group_by = GroupBy
-    reduce = Reduce
     aggregate = staticmethod(Aggregate)
 
     join = staticmethod(join)
     LEFT = _JoinConditions.LEFT
     RIGHT = _JoinConditions.RIGHT
 
-    Mut = Mutations
+    def reduce(self, to_call_with_2_args, *args, **kwargs):
+        if isinstance(to_call_with_2_args, type) and issubclass(
+            to_call_with_2_args, BaseReducer
+        ):
+            return to_call_with_2_args(*args, **kwargs)
+        return Reduce(to_call_with_2_args, *args, **kwargs)
 
     def __call__(self, obj: object):
         """Shortcut for ``ensure_conversion``"""
