@@ -1,8 +1,15 @@
+"""Helpers live here: like:
+ - recently used cache
+ - options ctx manager
+"""
+import threading
+import typing
 from collections import deque
-from threading import local
 
 
 class RUCache:
+    """Recently used cache - naive implementation"""
+
     def __init__(self, max_size, on_evict=None):
         self.max_size = max_size
         self._on_evict = on_evict
@@ -36,7 +43,7 @@ class RUCache:
         return False
 
     def _evict(self, expected_size):
-        for i in range(self.max_size + 1):
+        for _ in range(self.max_size + 1):
             if len(self._keys) > expected_size:
                 key = self._keys.pop()
                 if key in self._keys_to_be_put_back:
@@ -50,15 +57,24 @@ class RUCache:
                 break
 
 
-class BaseCtxMeta(type):
+if hasattr(typing, "GenericMeta"):
+    # python 3.6
+    GenericMeta = typing.GenericMeta
+else:
+    # python 3.7+
+    class GenericMeta(type):  # type: ignore
+        pass
+
+
+class BaseCtxMeta(GenericMeta):
     def __init__(cls, name, bases, kwargs):
-        super(BaseCtxMeta, cls).__init__(name, bases, kwargs)
-        cls._ctx = local()
+        super().__init__(name, bases, kwargs)
+        cls._ctx = threading.local()
 
 
 class BaseOptionsMeta(type):
     def __init__(cls, name, bases, kwargs):
-        super(BaseOptionsMeta, cls).__init__(name, bases, kwargs)
+        super().__init__(name, bases, kwargs)
         cls._option_attrs = {
             k: v
             for k, v in kwargs.items()
@@ -67,6 +83,8 @@ class BaseOptionsMeta(type):
 
 
 class BaseOptions(object, metaclass=BaseOptionsMeta):
+    """Container object, which carries current options"""
+
     def clone(self):
         clone = self.__class__()
         for option_attr in self._option_attrs.keys():
@@ -81,10 +99,16 @@ class BaseOptions(object, metaclass=BaseOptionsMeta):
                 setattr(self, option_attr, value)
 
 
-class BaseCtx(object, metaclass=BaseCtxMeta):
-    options_cls = None
+OT = typing.TypeVar("OT", bound=BaseOptions)
 
-    def __enter__(self) -> BaseOptions:
+
+class BaseCtx(typing.Generic[OT], metaclass=BaseCtxMeta):
+    """Context manager to manage option objects"""
+
+    options_cls: typing.Type[OT]
+    _ctx: threading.local
+
+    def __enter__(self) -> OT:
         self._ctx.prev_options = getattr(self._ctx, "options", None)
         if self._ctx.prev_options:
             self._ctx.options = self._ctx.prev_options.clone()
